/*
 * A page mapper providing memory pages to satisfy file accesses, masking the
 * limits of a visible region of the file's contents.
 *
 * Copyright (C) 2021, 2022 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA
 */

#pragma once

#include <fsserver/generic_page_mapper.h>
#include <mem/memory_incremental.h>



/* A special flexpage class for masked flexpages. */

class MaskedFlexpage : public Flexpage
{
public:
    explicit MaskedFlexpage(Region *region = NULL);
};



/* A file mapper, associating flexpages with file regions. */

class MaskedPageMapper : public GenericPageMapper
{
protected:
    GenericPageMapper *_mapper;
    offset_t _size;

    /* Masked region support. */

    MemoryIncremental _memory;
    offset_t _start_visible, _end_visible;
    MaskedFlexpage _start_flexpage, _end_flexpage, _zero_flexpage;

    /* Internal flexpage retrieval methods. */

    Flexpage *get_masked_flexpage(Flexpage *flexpage);

    void allocate_region(Flexpage *flexpage, Flexpage &masked);

    void populate_region(Flexpage *flexpage, Flexpage &masked,
                         bool has_start, bool has_end);

public:
    explicit MaskedPageMapper(GenericPageMapper *mapper, offset_t start_visible,
                              offset_t end_visible);

    virtual ~MaskedPageMapper();

    /* Interface for the pager, implementing GenericPageMapper. */

    Flexpage *get(offset_t offset, map_flags_t flags);

    void queue(Flexpage *flexpage);

    void flush_all(offset_t start, offset_t size);

    offset_t get_data_size();

    void set_data_size(offset_t size);
};

// vim: tabstop=4 expandtab shiftwidth=4
